@@ -3178,6 +3178,8 @@ OTF_check_features (OTF *otf, int gsubp,
 
   if (OTF_get_features (otf, gsubp) < 0)
     {
+      if (gsubp ? ! otf->gsub : ! otf->gpos)
+	return 0;
       for (i = 0; i < n_features; i++)
 	{
 	  OTF_Tag feature = features[i];
@@ -3187,7 +3189,6 @@ OTF_check_features (OTF *otf, int gsubp,
 	  if ((((unsigned) feature) & 0x80000000) == 0)
 	    return -1;
 	}
-      return 1;
     }
   if (gsubp)
     {
@@ -3209,8 +3210,10 @@ OTF_check_features (OTF *otf, int gsubp,
       for (i = 0; i < Script->LangSysCount && ! LangSys; i++)
 	if (Script->LangSysRecord[i].LangSysTag == language)
 	  LangSys = Script->LangSys + i;
+      if (! LangSys)
+	return 0;
     }
-  if (! LangSys)
+  else
     LangSys = &Script->DefaultLangSys;
   for (j = 0; j < n_features; j++)
     {
